window {
	background-image: url("/usr/share/nwg-hello/nwg.jpg"); background-size: auto 100%
}

#form-wrapper {
	background-color: rgba(0, 0, 0, 0.2)
}

entry {
	background-color: rgba(255, 255, 255, 0.1);
	border: 1px solid #eee;
	border-radius: 18px;
	padding: 12px
}

button {
	background: rgba(255, 255, 255, 0.1) none;
	border: 1px solid #eee;
	border-radius: 18px;
	padding: 12px
}

button:hover {
	background-color: rgba(255, 255, 255, 0.2)
}

#power-button {
	border-radius: 18px;
    background:none;
    border: none;
}

#power-button:hover {
    background-color: rgba(255, 255, 255, 0.1)
}

#power-button:active {
    background-color: rgba(255, 255, 0, 0.2)
}

#welcome-label {
	font-size: 48px
}

#clock-label {
	font-family: monospace;
	font-size: 30px
}

#date-label {
	font-size: 18px
}

#form-label {
}

#form-combo {
}

#password-entry {
}

#login-button {
}
